@@ -7,17 +7,15 @@ package Graph::AdjacencyMap::Vertex;
 use strict;
 use warnings;
 
-# $SIG{__DIE__ } = sub { use Carp; confess };
-# $SIG{__WARN__} = sub { use Carp; confess };
+# $SIG{__DIE__ } = \&Graph::__carp_confess;
+# $SIG{__WARN__} = \&Graph::__carp_confess;
 
 use Graph::AdjacencyMap qw(:flags :fields);
 use base 'Graph::AdjacencyMap';
 
-use Scalar::Util qw(weaken);
-
-sub _new {
-    my ($class, $flags, $arity) = @_;
-    bless [ 0, $flags, $arity ], $class;
+sub stringify {
+    require Graph::AdjacencyMap::Heavy;
+    goto &Graph::AdjacencyMap::Heavy::stringify;
 }
 
 require overload; # for de-overloading
@@ -30,91 +28,66 @@ sub __strval {
 }
 
 sub __set_path {
-    my $m = shift;
-    my $f = $m->[ _f ];
-    my $id = pop if ($f & _MULTI);
-    if (@_ != 1) {
-	require Carp;
-	Carp::confess(sprintf "Graph::AdjacencyMap::Vertex: arguments %d expected 1", scalar @_);
-    }
-    my $p;
-    $p = $m->[ _s ] ||= { };
-    my @p = $p;
-    my @k;
-    my $k = shift;
-    my $q = __strval($k, $f);
-    push @k, $q;
-    return (\@p, \@k);
+    my $f = $_[0]->[ _f ];
+    Graph::__carp_confess(sprintf "arguments %d expected %d for\n".$_[0]->stringify, @_ - 1)
+	if @_ != (2 + ($f & _MULTI));
+    [ $_[0]->[ _s ] ||= { } ], [ __strval($_[1], $f) ];
 }
 
 sub __set_path_node {
-    my ($m, $p, $l) = splice @_, 0, 3;
+    my ($m, $p, $l) = @_;
     my $f = $m->[ _f ];
-    my $id = pop if ($f & _MULTI);
-    unless (exists $p->[-1]->{ $l }) {
-	my $i = $m->_new_node( \$p->[-1]->{ $l }, $id );
-	$m->[ _i ]->{ defined $i ? $i : "" } = $_[0];
-    } else {
-	$m->_inc_node( \$p->[-1]->{ $l }, $id );
-    }
+    my $id = $_[-1] if ($f & _MULTI);
+    return $m->_inc_node( \$p->[-1]->{ $l }, $id ) if exists $p->[-1]->{ $l };
+    my $i = $m->_new_node( \$p->[-1]->{ $l }, $id );
+    die "undefined index" if !defined $i;
+    $m->[ _i ][ $i ] = $_[3];
 }
 
 sub set_path {
-    my $m = shift;
+    my ($m) = @_;
     my $f = $m->[ _f ];
-    my ($p, $k) = $m->__set_path( @_ );
+    my ($p, $k) = &__set_path;
     return unless defined $p && defined $k;
     my $l = defined $k->[-1] ? $k->[-1] : "";
-    my $set = $m->__set_path_node( $p, $l, @_ );
+    my $set = $m->__set_path_node( $p, $l, @_[1..$#_] );
     return $set;
 }
 
 sub __has_path {
-    my $m = shift;
-    my $f = $m->[ _f ];
-    if (@_ != 1) {
-	require Carp;
-	Carp::confess(sprintf
-		      "Graph::AdjacencyMap: arguments %d expected 1\n",
-		      scalar @_);
-    }
-    my $p = $m->[ _s ];
-    return unless defined $p;
-    my @p = $p;
-    my @k;
-    my $k = shift;
-    my $q = __strval($k, $f);
-    push @k, $q;
-    return (\@p, \@k);
+    my $f = $_[0]->[ _f ];
+    Graph::__carp_confess(sprintf "arguments %d expected %d for\n".$_[0]->stringify, @_ - 1, (1 + ($f & _MULTI)))
+	if @_ - 1 != (1 + ($f & _MULTI));
+    return unless defined(my $p = $_[0]->[ _s ]);
+    return ([$p], [ __strval($_[1], $f) ]);
 }
 
 sub has_path {
-    my $m = shift;
-    my ($p, $k) = $m->__has_path( @_ );
-    return unless defined $p && defined $k;
+    my $m = $_[0];
+    return unless my ($p, $k) = &__has_path;
     return exists $p->[-1]->{ defined $k->[-1] ? $k->[-1] : "" };
 }
 
 sub has_path_by_multi_id {
-    my $m = shift;
+    my ($m) = @_;
     my $id = pop;
-    my ($e, $n) = $m->__get_path_node( @_ );
+    my ($e, $n) = &{ $_[0]->can('__get_path_node') };
     return undef unless $e;
     return exists $n->[ _nm ]->{ $id };
 }
 
 sub _get_path_id {
-    my $m = shift;
+    my ($m) = @_;
     my $f = $m->[ _f ];
-    my ($e, $n) = $m->__get_path_node( @_ );
+    my ($e, $n) = &{ $_[0]->can('__get_path_node') };
     return undef unless $e;
     return ref $n ? $n->[ _ni ] : $n;
 }
 
 sub _get_path_count {
-    my $m = shift;
+    my ($m) = @_;
     my $f = $m->[ _f ];
-    my ($e, $n) = $m->__get_path_node( @_ );
+    my ($e, $n) = &{ $_[0]->can('__get_path_node') };
     return 0 unless $e && defined $n;
     return
 	($f & _COUNT) ? $n->[ _nc ] :
@@ -122,28 +95,31 @@ sub _get_path_count {
 }
 
 sub __attr {
-    my $m = shift;
-    if (@_ && ref $_[0] && @{ $_[0] } != $m->[ _a ]) {
-	require Carp;
-	Carp::confess(sprintf "Graph::AdjacencyMap::Vertex: arguments %d expected %d",
-		      scalar @{ $_[0] }, $m->[ _a ]);
-    }
+    my ($m) = @_;
+    return if @_ < 3;
+    Graph::__carp_confess(sprintf "arguments %d expected %d for\n".$m->stringify,
+		  @_ - 1, $m->[ _a ])
+        if @_ - 1 != $m->[ _a ];
+    my $f = $m->[ _f ];
+    return if !($f & _UNORDUNIQ);
+    goto &Graph::AdjacencyMap::__arg if ($f & _UNORDUNIQ) != _UNORD;
+    @_ = ($_[0], sort @_[1..$#_]);
 }
 
 sub _get_id_path {
     my ($m, $i) = @_;
-    return defined $m->[ _i ] ? $m->[ _i ]->{ $i } : undef;
+    return defined $m->[ _i ] ? $m->[ _i ][ $i ] : undef;
 }
 
 sub del_path {
-    my $m = shift;
+    my ($m) = @_;
     my $f = $m->[ _f ];
-    my ($e, $n, $p, $k, $l) = $m->__get_path_node( @_ );
+    my ($e, $n, $p, $k, $l) = &{ $_[0]->can('__get_path_node') };
     return unless $e;
     my $c = ($f & _COUNT) ? --$n->[ _nc ] : 0;
     if ($c == 0) {
-	delete $m->[ _i ]->{ ref $n ? $n->[ _ni ] : $n };
-	delete $p->[ -1 ]->{ $l };
+	delete $m->[ _i ][ ref $n ? $n->[ _ni ] : $n ];
+	delete $p->[ -1 ]{ $l };
     }
     return 1;
 }
@@ -152,28 +128,28 @@ sub rename_path {
     my ($m, $from, $to) = @_;
     my ($e, $n, $p, $k, $l) = $m->__get_path_node( $from );
     return unless $e;
-    $m->[ _i ]{ ref $n ? $n->[ _ni ] : $n } = $to;
-    $p->[ -1 ]->{ $to } = delete $p->[ -1 ]->{ $l };
+    $m->[ _i ][ ref $n ? $n->[ _ni ] : $n ] = $to;
+    $p->[ -1 ]{ $to } = delete $p->[ -1 ]{ $l };
     return 1;
 }
 
 sub del_path_by_multi_id {
-    my $m = shift;
+    my ($m) = @_;
     my $f = $m->[ _f ];
-    my $id = pop;
-    my ($e, $n, $p, $k, $l) = $m->__get_path_node( @_ );
+    my $id = $_[-1];
+    my ($e, $n, $p, $k, $l) = &{ $_[0]->can('__get_path_node') };
     return unless $e;
     delete $n->[ _nm ]->{ $id };
     unless (keys %{ $n->[ _nm ] }) {
-	delete $m->[ _i ]->{ $n->[ _ni ] };
-	delete $p->[-1]->{ $l };
+	delete $m->[ _i ][ $n->[ _ni ] ];
+	delete $p->[-1]{ $l };
     }
     return 1;
 }
 
 sub paths {
     my $m = shift;
-    return map { [ $_ ] } values %{ $m->[ _i ] } if defined $m->[ _i ];
+    return map [ $_ ], grep defined, @{ $m->[ _i ] } if defined $m->[ _i ];
     wantarray ? ( ) : 0;
 }
 
@@ -182,7 +158,7 @@ sub paths {
 
 =head1 NAME
 
-Graph::AdjacencyMap - create and a map of graph vertices or edges
+Graph::AdjacencyMap::Vertex - create and a map of graph vertices or edges
 
 =head1 SYNOPSIS
 
@@ -210,7 +186,7 @@ Return true if the Map has the path by ids, false if not.
 
 =item has_path_by_multi_id($id)
 
-Return true ifd the a Map has the path by a multi(vertex) id, false if not.
+Return true if the Map has the path by a multi(vertex) id, false if not.
 
 =item paths
 
