@@ -3,8 +3,18 @@ package Graph::Attribute;
 use strict;
 use warnings;
 
+my @API = qw(get_attribute
+	     get_attributes
+	     set_attribute
+	     set_attributes
+	     has_attribute
+	     has_attributes
+	     delete_attribute
+	     delete_attributes
+	     get_attribute_names
+	     get_attribute_values);
+
 sub _F () { 0 }
-sub _COMPAT02 () { 0x00000001 }
 
 sub import {
     my $package = shift;
@@ -31,21 +41,12 @@ sub import {
 	*{"${caller}::_has_attributes"} = sub { defined $_[0]->{ $k } };
 	*{"${caller}::_delete_attributes"} = sub { delete $_[0]->{ $k } };
     } else {
+	# uncoverable statement
 	die "Graph::Attribute::import($package @_) caller $caller\n";
     }
-    my @api = qw(get_attribute
-		 get_attributes
-		 set_attribute
-		 set_attributes
-		 has_attribute
-		 has_attributes
-		 delete_attribute
-		 delete_attributes
-		 get_attribute_names
-		 get_attribute_values);
     if (exists $attr{map}) {
 	my $map = $attr{map};
-	for my $api (@api) {
+	for my $api (@API) {
 	    my ($first, $rest) = ($api =~ /^(\w+?)_(.+)/);
 	    no strict 'refs';
 	    *{"${caller}::${first}_${map}_${rest}"} = \&$api;
@@ -91,23 +92,16 @@ sub get_attribute {
 sub delete_attribute {
     my $g = shift;
     my $a = pop;
-    my $p = $g->_get_attributes;
-    if (defined $p) {
-	delete $p->{ $a };
-	return 1;
-    } else {
-	return 0;
-    }
+    return 0 unless defined(my $p = $g->_get_attributes);
+    delete $p->{ $a };
+    return 1;
 }
 
 sub delete_attributes {
     my $g = shift;
-    if ($g->_has_attributes) {
-	$g->_delete_attributes;
-	return 1;
-    } else {
-	return 0;
-    }
+    return 0 if !$g->_has_attributes;
+    $g->_delete_attributes;
+    return 1;
 }
 
 sub get_attribute_names {
@@ -123,9 +117,7 @@ sub get_attribute_values {
 }
 
 sub get_attributes {
-    my $g = shift;
-    my $a = $g->_get_attributes;
-    ($g->[ _F ] & _COMPAT02) ? (defined $a ? %{ $a } : ()) : $a;
+    $_[0]->_get_attributes;
 }
 
 1;
