@@ -3,8 +3,8 @@ package Graph::Traversal;
 use strict;
 use warnings;
 
-# $SIG{__DIE__ } = sub { use Carp; confess };
-# $SIG{__WARN__} = sub { use Carp; confess };
+# $SIG{__DIE__ } = \&Graph::__carp_confess;
+# $SIG{__WARN__} = \&Graph::__carp_confess;
 
 sub DEBUG () { 0 }
 
@@ -61,25 +61,8 @@ sub configure {
     $self->{ post_vertex } = $attr{ post_vertex } if exists $attr{ post_vertex };
     $self->{ pre_edge  } = $attr{ pre_edge  } if exists $attr{ pre_edge  };
     $self->{ post_edge } = $attr{ post_edge } if exists $attr{ post_edge };
-    if (exists $attr{ successor }) { # Graph 0.201 compatibility.
-	$self->{ tree_edge } = $self->{ non_tree_edge } = $attr{ successor };
-    }
-    if (exists $attr{ unseen_successor }) {
-	if (exists $self->{ tree_edge }) { # Graph 0.201 compatibility.
-	    my $old_tree_edge = $self->{ tree_edge };
-	    $self->{ tree_edge } = sub {
-		$old_tree_edge->( @_ );
-		$attr{ unseen_successor }->( @_ );
-	    };
-	} else {
-	    $self->{ tree_edge } = $attr{ unseen_successor };
-	}
-    }
     if ($self->graph->multiedged || $self->graph->countedged) {
 	$self->{ seen_edge } = $attr{ seen_edge } if exists $attr{ seen_edge };
-	if (exists $attr{ seen_successor }) { # Graph 0.201 compatibility.
-	    $self->{ seen_edge } = $attr{ seen_edge };
-	}
     }
     $self->{ non_tree_edge } = $attr{ non_tree_edge } if exists $attr{ non_tree_edge };
     $self->{ pre_edge  } = $attr{ tree_edge } if exists $attr{ tree_edge };
@@ -90,9 +73,6 @@ sub configure {
 	$attr{ first_root } = $attr{ start };
 	$attr{ next_root  } = undef;
     }
-    if (exists $attr{ get_next_root }) {
-	$attr{ next_root  } = $attr{ get_next_root }; # Graph 0.201 compat.
-    }
     $self->{ next_root } =
 	exists $attr{ next_root } ?
 	    $attr{ next_root } :
@@ -141,28 +121,24 @@ sub configure {
     $self->{ see } = $see;
     delete @attr{ qw(
 		     pre post pre_edge post_edge
-		     successor unseen_successor seen_successor
 		     tree_edge non_tree_edge
 		     back_edge down_edge cross_edge seen_edge
-		     start get_next_root
+		     start
 		     next_root next_alphabetic next_numeric next_random next_successor
 		     first_root
 		     has_a_cycle find_a_cycle
 		    ) };
     if (keys %attr) {
-	require Carp;
 	my @attr = sort keys %attr;
-	Carp::croak(sprintf "Graph::Traversal: unknown attribute%s @{[map { qq['$_'] } @attr]}\n", @attr == 1 ? '' : 's');
+	Graph::__carp_confess(sprintf "Graph::Traversal: unknown attribute%s @{[map { qq['$_'] } @attr]}\n", @attr == 1 ? '' : 's');
     }
 }
 
 sub new {
     my $class = shift;
     my $g = shift;
-    unless (ref $g && $g->isa('Graph')) {
-	require Carp;
-	Carp::croak("Graph::Traversal: first argument is not a Graph");
-    }
+    Graph::__carp_confess("Graph::Traversal: first argument is not a Graph")
+	unless ref $g && $g->isa('Graph');
     my $self = { graph => $g, state => { } };
     bless $self, $class;
     $self->reset;
@@ -290,9 +266,7 @@ sub next {
 	@next = values %next;
 	my @all = @next;
 	print "all = @all\n" if DEBUG;
-	for my $s (keys %next) {
-	    delete $next{$s} if exists $self->{seen}->{$s};
-	}
+	delete @next{ grep exists $self->{seen}{$_}, keys %next };
 	@next = values %next;
 	print "next.2 - @next\n" if DEBUG;
 	if (@next) {
@@ -346,12 +320,7 @@ sub next {
 sub _order {
     my ($self, $order) = @_;
     1 while defined $self->next;
-    my $wantarray = wantarray;
-    if ($wantarray) {
-	@{ $self->{ $order } };
-    } elsif (defined $wantarray) {
-	shift @{ $self->{ $order } };
-    }
+    @{ $self->{ $order } };
 }
 
 sub preorder {
@@ -670,31 +639,6 @@ Delete the state 's' from the traversal.
 
 =back
 
-=head2 Backward compatibility
-
-The following parameters are for backward compatibility to Graph 0.2xx:
-
-=over 4
-
-=item get_next_root
-
-Like C<next_root>.
-
-=item successor
-
-Identical to having C<tree_edge> both C<non_tree_edge> defined
-to be the same.
-
-=item unseen_successor
-
-Like C<tree_edge>.
-
-=item seen_successor
-
-Like C<seed_edge>.
-
-=back
-
 =head2 Special callbacks
 
 If in a callback you call the special C<terminate> method,
